#!/usr/bin/env python3.0

import unittest
import cgi
from io import StringIO, BytesIO, TextIOWrapper


noFileASCII = 'nofileASCII.dat'
noFileNonASCII = 'nofileNonASCII.dat'
ASCIIFile = 'ASCII.dat'
NonASCIIFile = 'UTF8.dat'
BinaryFile = 'Binary.dat'

def unifyNewlines(input):
    return input.replace(b'\r\n', b'\n').replace(b'\r', b'\n')

class TestFieldStorage(unittest.TestCase):

    def setup(self, fn):
        fp = open(fn, 'rb')
        fp.encoding = 'latin-1'
        content_type = fp.readline().split(b': ')[1].rstrip().decode('ascii')
        content_length = fp.readline().split(b': ')[1].rstrip().decode('ascii')
        fp.readline() # blank line
        env = dict(CONTENT_LENGTH=content_length,
                        CONTENT_TYPE=content_type,
                        REQUEST_METHOD="POST")
        self.fp = fp
        return cgi.FieldStorage(fp=fp, environ=env)

    def tearDown(self):
        self.fp.close()

    def testNormalForm(self):
        fs = self.setup(noFileASCII)
        self.assertTrue('upkey' in fs.keys())
        self.assertEqual(fs.getvalue('upkey'), 'Now is the time for all good men to come to the aide of their country.')
        self.assertTrue('submit' in fs.keys())
        self.assertEqual(fs.getvalue('submit'), 'submit')

    def testNonASCIIForm(self):
        fs = self.setup(noFileNonASCII)
        self.assertTrue('upkey' in fs.keys())
        self.assertEqual(fs.getvalue('upkey'), '\xd0\xa1\xd0\xb5\xd0\xb9\xd1\x87\xd0\xb0\xd1\x81 \xd0\xbd\xd0\xb0\xd1\x81\xd1\x82\xd0\xb0\xd0\xbb\xd0\xbe \xd0\xb2\xd1\x80\xd0\xb5\xd0\xbc\xd1\x8f \xd0\xb4\xd0\xbb\xd1\x8f \xd0\xb2\xd1\x81\xd0\xb5\xd1\x85 \xd1\x85\xd0\xbe\xd1\x80\xd0\xbe\xd1\x88\xd0\xb8\xd1\x85 \xd0\xbb\xd1\x8e\xd0\xb4\xd0\xb5\xd0\xb9 \xd0\xbf\xd1\x80\xd0\xb8\xd0\xb9\xd1\x82\xd0\xb8 \xd0\xba \xd0\xbf\xd0\xb0\xd0\xbc\xd1\x8f\xd1\x82\xd0\xbd\xd0\xbe\xd0\xb9 \xd1\x81\xd0\xb2\xd0\xbe\xd0\xb5\xd0\xb9 \xd1\x81\xd1\x82\xd1\x80\xd0\xb0\xd0\xbd\xd1\x8b.')
        self.assertTrue('submit' in fs.keys())
        self.assertEqual(fs.getvalue('submit'), 'submit')

    def testASCIIFileUploadForm(self):
        src = open('zenASCII.txt', 'rb')
        expect = unifyNewlines(src.read())
        src.close()
        fs = self.setup(ASCIIFile)
        self.assertTrue('upkey' in fs.keys())
        self.assertEqual(fs.getvalue('upkey'), 'test upkey')
        self.assertTrue('submit' in fs.keys())
        self.assertEqual(fs.getvalue('submit'), 'submit')

        self.assertTrue('upfile' in fs.keys())
        self.assertEqual(fs['upfile'].filename, 'zenASCII.txt')
        self.assertEqual(unifyNewlines(fs.getvalue('upfile')),expect)

    def testNonASCIIFileUploadForm(self):
        src = open('zenUTF8-RU.txt', 'rb')
        expect = unifyNewlines(src.read())
        src.close()
        fs = self.setup(NonASCIIFile)
        self.assertTrue('upkey' in fs.keys())
        self.assertEqual(fs.getvalue('upkey'), 'test upkey')
        self.assertTrue('submit' in fs.keys())
        self.assertEqual(fs.getvalue('submit'), 'submit')

        self.assertTrue('upfile' in fs.keys())
        self.assertEqual(fs['upfile'].filename, 'zenUTF8-RU.txt')
        self.assertEqual(unifyNewlines(fs.getvalue('upfile')),expect)

    def testBinaryFileUploadForm(self):
        src = open('zenBinary.zip', 'rb')
        expect = src.read()
        src.close()
        fs = self.setup(BinaryFile)
        self.assertTrue('upkey' in fs.keys())
        self.assertEqual(fs.getvalue('upkey'), 'test upkey')
        self.assertTrue('submit' in fs.keys())
        self.assertEqual(fs.getvalue('submit'), 'submit')

        self.assertTrue('upfile' in fs.keys())
        self.assertEqual(fs['upfile'].filename, 'zenBinary.zip')
        self.assertEqual(fs.getvalue('upfile'),expect)

if __name__ == "__main__":
    unittest.main()
